@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: dialogs.h 8846 2009-07-22 15:55:48Z charles $
+ * $Id: dialogs.h 13625 2012-12-05 17:29:46Z jordan $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -22,26 +22,18 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-#ifndef TG_PREFS_H
-#define TG_PREFS_H
+#ifndef GTR_DIALOGS_H
+#define GTR_DIALOGS_H
 
 #include "tr-core.h"
-#include "tr-torrent.h"
 #include "util.h"
 
+/**
+ * Prompt the user to confirm removing a torrent.
+ */
+void gtr_confirm_remove (GtkWindow * parent,
+                          TrCore    * core,
+                          GSList    * gtorrents,
+                          gboolean    doDelete);
 
-/* used for a callback function with a data parameter */
-typedef void ( *callbackfunc_t )( gpointer );
-
-/* prompt if the user wants to quit, calls func with cbdata if they do */
-void askquit( TrCore          * core,
-              GtkWindow       * parent,
-              callbackfunc_t    func,
-              gpointer          cbdata );
-
-void confirmRemove( GtkWindow * parent,
-                    TrCore    * core,
-                    GSList    * gtorrents,
-                    gboolean    doDelete );
-
-#endif /* TG_PREFS_H */
+#endif /* GTR_DIALOGS_H */
