@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: BlocklistDownloaderViewController.m 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id: BlocklistDownloaderViewController.m 13253 2012-03-13 03:20:09Z livings124 $
  *
- * Copyright (c) 2008-2009 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -37,21 +37,25 @@
 
 @implementation BlocklistDownloaderViewController
 
+BlocklistDownloaderViewController * fBLViewController = nil;
 + (void) downloadWithPrefsController: (PrefsController *) prefsController
 {
-    BlocklistDownloaderViewController * downloader = [[BlocklistDownloaderViewController alloc] initWithPrefsController: prefsController];
-    [downloader startDownload];
+    if (!fBLViewController)
+    {
+        fBLViewController = [[BlocklistDownloaderViewController alloc] initWithPrefsController: prefsController];
+        [fBLViewController startDownload];
+    }
 }
 
 - (void) awakeFromNib
 {
     [fButton setTitle: NSLocalizedString(@"Cancel", "Blocklist -> cancel button")];
     
-    CGFloat oldWidth = [fButton frame].size.width;
+    const CGFloat oldWidth = NSWidth([fButton frame]);
     [fButton sizeToFit];
     NSRect buttonFrame = [fButton frame];
-    buttonFrame.size.width += 12.0f; //sizeToFit sizes a bit too small
-    buttonFrame.origin.x -= buttonFrame.size.width - oldWidth;
+    buttonFrame.size.width += 12.0; //sizeToFit sizes a bit too small
+    buttonFrame.origin.x -= NSWidth(buttonFrame) - oldWidth;
     [fButton setFrame: buttonFrame];
     
     [fProgressBar setUsesThreadedAnimation: YES];
@@ -76,8 +80,7 @@
     {
         [fProgressBar setIndeterminate: NO];
         
-        NSString * substring = [NSString stringWithFormat: NSLocalizedString(@"%@ of %@", "Blocklist -> message"),
-                                [NSString stringForFileSize: currentSize], [NSString stringForFileSize: expectedSize]];
+        NSString * substring = [NSString stringForFilePartialSize: currentSize fullSize: expectedSize];
         string = [string stringByAppendingFormat: @" (%@)",  substring];
         [fProgressBar setDoubleValue: (double)currentSize / expectedSize];
     }
@@ -102,6 +105,7 @@
     [NSApp endSheet: fStatusWindow];
     [fStatusWindow orderOut: self];
     
+    fBLViewController = nil;
     [self release];
 }
 
@@ -115,8 +119,7 @@
     [alert setMessageText: NSLocalizedString(@"Download of the blocklist failed.", "Blocklist -> message")];
     [alert setAlertStyle: NSWarningAlertStyle];
     
-    [alert setInformativeText: [NSString stringWithFormat: @"%@ - %@", NSLocalizedString(@"Error", "Blocklist -> message"),
-        error]];
+    [alert setInformativeText: error];
     
     [alert beginSheetModalForWindow: [fPrefsController window] modalDelegate: self
         didEndSelector: @selector(failureSheetClosed:returnCode:contextInfo:) contextInfo: nil];
@@ -140,15 +143,18 @@
 {
     //load window and show as sheet
     [NSBundle loadNibNamed: @"BlocklistStatusWindow" owner: self];
-    [NSApp beginSheet: fStatusWindow modalForWindow: [fPrefsController window] modalDelegate: nil didEndSelector: nil contextInfo: nil];
     
     BlocklistDownloader * downloader = [BlocklistDownloader downloader];
-    [downloader setViewController: self];
+    [downloader setViewController: self]; //do before showing the sheet to ensure it doesn't slide out with placeholder text
+    
+    [NSApp beginSheet: fStatusWindow modalForWindow: [fPrefsController window] modalDelegate: nil didEndSelector: nil contextInfo: nil];
 }
 
 - (void) failureSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info
 {
     [[alert window] orderOut: self];
+    
+    fBLViewController = nil;
     [self release];
 }
 
