@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: CreatorWindowController.m 9374 2009-10-22 03:31:59Z livings124 $
+ * $Id: CreatorWindowController.m 14662 2016-01-06 11:05:37Z mikedld $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,18 +23,21 @@
  *****************************************************************************/
 
 #import "CreatorWindowController.h"
+#import "Controller.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
-#import "utils.h" //tr_httpIsValidURL
+
+#import "transmission.h" // required by utils.h
+#import "utils.h" // tr_urlIsValidTracker()
 
 #define TRACKER_ADD_TAG 0
 #define TRACKER_REMOVE_TAG 1
 
 @interface CreatorWindowController (Private)
 
-+ (NSString *) chooseFile;
-- (void) updateEnableOpenCheckForTrackers;
-- (void) locationSheetClosed: (NSSavePanel *) openPanel returnCode: (NSInteger) code contextInfo: (void *) info;
++ (NSURL *) chooseFile;
 
+- (void) updateLocationField;
 - (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo;
 - (void) createReal;
 - (void) checkProgress;
@@ -44,31 +47,33 @@
 
 @implementation CreatorWindowController
 
-+ (void) createTorrentFile: (tr_session *) handle
++ (CreatorWindowController *) createTorrentFile: (tr_session *) handle
 {
     //get file/folder for torrent
-    NSString * path;
+    NSURL * path;
     if (!(path = [CreatorWindowController chooseFile]))
-        return;
+        return nil;
     
     CreatorWindowController * creator = [[self alloc] initWithHandle: handle path: path];
     [creator showWindow: nil];
+    return creator;
 }
 
-+ (void) createTorrentFile: (tr_session *) handle forFile: (NSString *) file
++ (CreatorWindowController *) createTorrentFile: (tr_session *) handle forFile: (NSURL *) file
 {
     CreatorWindowController * creator = [[self alloc] initWithHandle: handle path: file];
     [creator showWindow: nil];
+    return creator;
 }
 
-- (id) initWithHandle: (tr_session *) handle path: (NSString *) path
+- (id) initWithHandle: (tr_session *) handle path: (NSURL *) path
 {
     if ((self = [super initWithWindowNibName: @"Creator"]))
     {
         fStarted = NO;
         
         fPath = [path retain];
-        fInfo = tr_metaInfoBuilderCreate([fPath UTF8String]);
+        fInfo = tr_metaInfoBuilderCreate([[fPath path] UTF8String]);
         
         if (fInfo->fileCount == 0)
         {
@@ -108,7 +113,7 @@
         //get list of trackers
         if (!(fTrackers = [[fDefaults arrayForKey: @"CreatorTrackers"] mutableCopy]))
         {
-            fTrackers = [[NSMutableArray alloc] initWithCapacity: 1];
+            fTrackers = [[NSMutableArray alloc] init];
             
             //check for single tracker from versions before 1.3
             NSString * tracker;
@@ -126,7 +131,7 @@
         //remove potentially invalid addresses
         for (NSInteger i = [fTrackers count]-1; i >= 0; i--)
         {
-            if (!tr_httpIsValidURL([[fTrackers objectAtIndex: i] UTF8String]))
+            if (!tr_urlIsValidTracker([[fTrackers objectAtIndex: i] UTF8String]))
                 [fTrackers removeObjectAtIndex: i];
         }
     }
@@ -135,17 +140,19 @@
 
 - (void) awakeFromNib
 {
+    [[self window] setRestorationClass: [self class]];
+    
     NSString * name = [fPath lastPathComponent];
     
     [[self window] setTitle: name];
     
     [fNameField setStringValue: name];
-    [fNameField setToolTip: fPath];
+    [fNameField setToolTip: [fPath path]];
     
-    BOOL multifile = !fInfo->isSingleFile;
+    const BOOL multifile = fInfo->isFolder;
     
     NSImage * icon = [[NSWorkspace sharedWorkspace] iconForFileType: multifile
-                        ? NSFileTypeForHFSTypeCode('fldr') : [fPath pathExtension]];
+                        ? NSFileTypeForHFSTypeCode(kGenericFolderIcon) : [fPath pathExtension]];
     [icon setSize: [fIconView frame].size];
     [fIconView setImage: icon];
     
@@ -155,34 +162,36 @@
         NSString * fileString;
         NSInteger count = fInfo->fileCount;
         if (count != 1)
-            fileString = [NSString stringWithFormat: NSLocalizedString(@"%d files", "Create torrent -> info"), count];
+            fileString = [NSString stringWithFormat: NSLocalizedString(@"%@ files", "Create torrent -> info"),
+                            [NSString formattedUInteger: count]];
         else
             fileString = NSLocalizedString(@"1 file", "Create torrent -> info");
         statusString = [NSString stringWithFormat: @"%@, %@", fileString, statusString];
     }
     [fStatusField setStringValue: statusString];
     
-    NSString * piecesCountString;
-    int piecesCount = fInfo->pieceCount;
-    if (piecesCount == 1)
-        piecesCountString = NSLocalizedString(@"1 piece", "Create torrent -> info");
+    if (fInfo->pieceCount == 1)
+        [fPiecesField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"1 piece, %@", "Create torrent -> info"),
+                                                            [NSString stringForFileSize: fInfo->pieceSize]]];
     else
-        piecesCountString = [NSString stringWithFormat: NSLocalizedString(@"%d pieces", "Create torrent -> info"),
-                                                                piecesCount];
-    [fPiecesField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@, %@ each", "Create torrent -> info"),
-                                        piecesCountString, [NSString stringForFileSize: fInfo->pieceSize]]];
+        [fPiecesField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%d pieces, %@ each", "Create torrent -> info"),
+                                                            fInfo->pieceCount, [NSString stringForFileSize: fInfo->pieceSize]]];
     
-    fLocation = [[[[fDefaults stringForKey: @"CreatorLocation"] stringByExpandingTildeInPath] stringByAppendingPathComponent:
-                    [name stringByAppendingPathExtension: @"torrent"]] retain];
-    [fLocationField setStringValue: [fLocation stringByAbbreviatingWithTildeInPath]];
-    [fLocationField setToolTip: fLocation];
+    fLocation = [[[fDefaults URLForKey: @"CreatorLocationURL"] URLByAppendingPathComponent: [name stringByAppendingPathExtension: @"torrent"]] retain];
+    if (!fLocation)
+    {
+        //for 2.5 and earlier
+        #warning we still store "CreatorLocation" in Defaults.plist, and not "CreatorLocationURL"
+        NSString * location = [fDefaults stringForKey: @"CreatorLocation"];
+        fLocation = [[NSURL alloc] initFileURLWithPath: [[location stringByExpandingTildeInPath] stringByAppendingPathComponent: [name stringByAppendingPathExtension: @"torrent"]]];
+    }
+    [self updateLocationField];
     
     //set previously saved values
     if ([fDefaults objectForKey: @"CreatorPrivate"])
         [fPrivateCheck setState: [fDefaults boolForKey: @"CreatorPrivate"] ? NSOnState : NSOffState];
     
-    fOpenTorrent = [fDefaults boolForKey: @"CreatorOpen"];
-    [self updateEnableOpenCheckForTrackers];
+    [fOpenCheck setState: [fDefaults boolForKey: @"CreatorOpen"] ? NSOnState : NSOffState];
 }
 
 - (void) dealloc
@@ -196,16 +205,50 @@
         tr_metaInfoBuilderFree(fInfo);
     
     [fTimer invalidate];
+    [fTimer release];
     
     [super dealloc];
 }
 
-- (void) toggleOpenCheck: (id) sender
++ (void) restoreWindowWithIdentifier: (NSString *) identifier state: (NSCoder *) state completionHandler: (void (^)(NSWindow *, NSError *)) completionHandler
 {
-    fOpenTorrent = [fOpenCheck state] == NSOnState;
+    NSURL * path = [state decodeObjectForKey: @"TRCreatorPath"];
+    if (!path || ![path checkResourceIsReachableAndReturnError: nil])
+    {
+        completionHandler(nil, [NSError errorWithDomain: NSURLErrorDomain code: NSURLErrorCannotOpenFile userInfo: nil]);
+        return;
+    }
+    
+    NSWindow * window = [[self createTorrentFile: [(Controller *)[NSApp delegate] sessionHandle] forFile: path] window];
+    completionHandler(window, nil);
 }
 
-- (void) setLocation: (id) sender
+- (void) window: (NSWindow *) window willEncodeRestorableState: (NSCoder *) state
+{
+    [state encodeObject: fPath forKey: @"TRCreatorPath"];
+    [state encodeObject: fLocation forKey: @"TRCreatorLocation"];
+    [state encodeObject: fTrackers forKey: @"TRCreatorTrackers"];
+    [state encodeInteger: [fOpenCheck state] forKey: @"TRCreatorOpenCheck"];
+    [state encodeInteger: [fPrivateCheck state] forKey: @"TRCreatorPrivateCheck"];
+    [state encodeObject: [fCommentView string] forKey: @"TRCreatorPrivateComment"];
+}
+
+- (void) window: (NSWindow *) window didDecodeRestorableState: (NSCoder *) coder
+{
+    [fLocation release];
+    fLocation = [[coder decodeObjectForKey: @"TRCreatorLocation"] retain];
+    [self updateLocationField];
+    
+    [fTrackers release];
+    fTrackers = [[coder decodeObjectForKey: @"TRCreatorTrackers"] retain];
+    [fTrackerTable reloadData];
+    
+    [fOpenCheck setState: [coder decodeIntegerForKey: @"TRCreatorOpenCheck"]];
+    [fPrivateCheck setState: [coder decodeIntegerForKey: @"TRCreatorPrivateCheck"]];
+    [fCommentView setString: [coder decodeObjectForKey: @"TRCreatorPrivateComment"]];
+}
+
+- (IBAction) setLocation: (id) sender
 {
     NSSavePanel * panel = [NSSavePanel savePanel];
 
@@ -216,24 +259,41 @@
     [panel setAllowedFileTypes: [NSArray arrayWithObjects: @"org.bittorrent.torrent", @"torrent", nil]];
     [panel setCanSelectHiddenExtension: YES];
     
-    [panel beginSheetForDirectory: [fLocation stringByDeletingLastPathComponent] file: [fLocation lastPathComponent]
-            modalForWindow: [self window] modalDelegate: self didEndSelector: @selector(locationSheetClosed:returnCode:contextInfo:)
-            contextInfo: nil];
+    [panel setDirectoryURL: [fLocation URLByDeletingLastPathComponent]];
+    [panel setNameFieldStringValue: [fLocation lastPathComponent]];
+    
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        if (result == NSFileHandlingPanelOKButton)
+        {
+            [fLocation release];
+            fLocation = [[panel URL] retain];
+            [self updateLocationField];
+        }
+    }];
 }
 
-- (void) create: (id) sender
+- (IBAction) create: (id) sender
 {
     //make sure the trackers are no longer being verified
     if ([fTrackerTable editedRow] != -1)
         [[self window] endEditingFor: fTrackerTable];
     
-    if ([fTrackers count] == 0 && [fDefaults boolForKey: @"WarningCreatorBlankAddress"])
+    const BOOL isPrivate = [fPrivateCheck state] == NSOnState;
+    if ([fTrackers count] == 0
+        && [fDefaults boolForKey: isPrivate ? @"WarningCreatorPrivateBlankAddress" : @"WarningCreatorBlankAddress"])
     {
         NSAlert * alert = [[NSAlert alloc] init];
-        [alert setMessageText: NSLocalizedString(@"There is no tracker address.", "Create torrent -> blank address -> title")];
-        [alert setInformativeText: NSLocalizedString(@"The torrent file will not be able to be opened."
-            " A torrent file with no tracker address is only useful when you plan to upload the file to a tracker website"
-            " that will add the address for you.", "Create torrent -> blank address -> message")];
+        [alert setMessageText: NSLocalizedString(@"There are no tracker addresses.", "Create torrent -> blank address -> title")];
+        
+        NSString * infoString = isPrivate
+                    ? NSLocalizedString(@"A transfer marked as private with no tracker addresses will be unable to connect to peers."
+                        " The torrent file will only be useful if you plan to upload the file to a tracker website"
+                        " that will add the addresses for you.", "Create torrent -> blank address -> message")
+                    : NSLocalizedString(@"The transfer will not contact trackers for peers, and will have to rely solely on"
+                        " non-tracker peer discovery methods such as PEX and DHT to download and seed.",
+                        "Create torrent -> blank address -> message");
+        
+        [alert setInformativeText: infoString];
         [alert addButtonWithTitle: NSLocalizedString(@"Create", "Create torrent -> blank address -> button")];
         [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Create torrent -> blank address -> button")];
         [alert setShowsSuppressionButton: YES];
@@ -245,17 +305,17 @@
         [self createReal];
 }
 
-- (void) cancelCreateWindow: (id) sender
+- (IBAction) cancelCreateWindow: (id) sender
 {
     [[self window] close];
 }
 
 - (void) windowWillClose: (NSNotification *) notification
 {
-    [self release];
+    [self autorelease];
 }
 
-- (void) cancelCreateProgress: (id) sender
+- (IBAction) cancelCreateProgress: (id) sender
 {
     fInfo->abortFlag = 1;
     [fTimer fire];
@@ -271,7 +331,7 @@
     return [fTrackers objectAtIndex: row];
 }
 
-- (void) addRemoveTracker: (id) sender
+- (IBAction) addRemoveTracker: (id) sender
 {
     //don't allow add/remove when currently adding - it leads to weird results
     if ([fTrackerTable editedRow] != -1)
@@ -283,8 +343,6 @@
         
         [fTrackerTable deselectAll: self];
         [fTrackerTable reloadData];
-        
-        [self updateEnableOpenCheckForTrackers];
     }
     else
     {
@@ -307,16 +365,13 @@
     if ([tracker rangeOfString: @"://"].location == NSNotFound)
         tracker = [@"http://" stringByAppendingString: tracker];
     
-    if (!tr_httpIsValidURL([tracker UTF8String]))
+    if (!tr_urlIsValidTracker([tracker UTF8String]))
     {
         NSBeep();
         [fTrackers removeObjectAtIndex: row];
     }
     else
-    {
         [fTrackers replaceObjectAtIndex: row withObject: tracker];
-        [self updateEnableOpenCheckForTrackers];
-    }
     
     [fTrackerTable deselectAll: self];
     [fTrackerTable reloadData];
@@ -327,11 +382,80 @@
     [fTrackerAddRemoveControl setEnabled: [fTrackerTable numberOfSelectedRows] > 0 forSegment: TRACKER_REMOVE_TAG];
 }
 
+- (void) copy: (id) sender
+{
+    NSArray * addresses = [fTrackers objectsAtIndexes: [fTrackerTable selectedRowIndexes]];
+    NSString * text = [addresses componentsJoinedByString: @"\n"];
+    
+    NSPasteboard * pb = [NSPasteboard generalPasteboard];
+    [pb clearContents];
+    [pb writeObjects: [NSArray arrayWithObject: text]];
+}
+
+- (BOOL) validateMenuItem: (NSMenuItem *) menuItem
+{
+    const SEL action = [menuItem action];
+    
+    if (action == @selector(copy:))
+        return [[self window] firstResponder] == fTrackerTable && [fTrackerTable numberOfSelectedRows] > 0;
+    
+    if (action == @selector(paste:))
+        return [[self window] firstResponder] == fTrackerTable
+            && [[NSPasteboard generalPasteboard] canReadObjectForClasses: [NSArray arrayWithObject: [NSString class]] options: nil];
+    
+    return YES;
+}
+
+- (void) paste: (id) sender
+{
+    NSMutableArray * tempTrackers = [NSMutableArray array];
+    
+    NSArray * items = [[NSPasteboard generalPasteboard] readObjectsForClasses: [NSArray arrayWithObject: [NSString class]] options: nil];
+    NSAssert(items != nil, @"no string items to paste; should not be able to call this method");
+    
+    for (NSString * pbItem in items)
+    {
+        for (NSString * tracker in [pbItem componentsSeparatedByString: @"\n"])
+            [tempTrackers addObject: tracker];
+    }
+    
+    BOOL added = NO;
+    
+    for (NSString * tracker in tempTrackers)
+    {
+        tracker = [tracker stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+        
+        if ([tracker rangeOfString: @"://"].location == NSNotFound)
+            tracker = [@"http://" stringByAppendingString: tracker];
+        
+        if (tr_urlIsValidTracker([tracker UTF8String]))
+        {
+            [fTrackers addObject: tracker];
+            added = YES;
+        }
+    }
+    
+    if (added)
+    {
+        [fTrackerTable deselectAll: self];
+        [fTrackerTable reloadData];
+    }
+    else
+        NSBeep();
+}
+
 @end
 
 @implementation CreatorWindowController (Private)
 
-+ (NSString *) chooseFile
+- (void) updateLocationField
+{
+    NSString * pathString = [fLocation path];
+    [fLocationField setStringValue: [pathString stringByAbbreviatingWithTildeInPath]];
+    [fLocationField setToolTip: pathString];
+}
+
++ (NSURL *) chooseFile
 {
     NSOpenPanel * panel = [NSOpenPanel openPanel];
     
@@ -345,32 +469,17 @@
     [panel setMessage: NSLocalizedString(@"Select a file or folder for the torrent file.", "Create torrent -> select file")];
     
     BOOL success = [panel runModal] == NSOKButton;
-    return success ? [[panel filenames] objectAtIndex: 0] : nil;
-}
-
-- (void) updateEnableOpenCheckForTrackers
-{
-    BOOL hasTracker = [fTrackers count] > 0;
-    [fOpenCheck setEnabled: hasTracker];
-    [fOpenCheck setState: (fOpenTorrent && hasTracker) ? NSOnState : NSOffState];
-}
-
-- (void) locationSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (void *) info
-{
-    if (code == NSOKButton)
-    {
-        [fLocation release];
-        fLocation = [[panel filename] retain];
-        
-        [fLocationField setStringValue: [fLocation stringByAbbreviatingWithTildeInPath]];
-        [fLocationField setToolTip: fLocation];
-    }
+    return success ? [[panel URLs] objectAtIndex: 0] : nil;
 }
 
 - (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo
 {
     if ([[alert suppressionButton] state] == NSOnState)
-        [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"];
+    {
+        [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"]; //set regardless of private/public
+        if ([fPrivateCheck state] == NSOnState)
+            [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorPrivateBlankAddress"];
+    }
     
     [alert release];
     
@@ -380,8 +489,26 @@
 
 - (void) createReal
 {
+    //check if the location currently exists
+    if (![[fLocation URLByDeletingLastPathComponent] checkResourceIsReachableAndReturnError: NULL])
+    {
+        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
+        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> directory doesn't exist warning -> button")];
+        [alert setMessageText: NSLocalizedString(@"The chosen torrent file location does not exist.",
+                                                "Create torrent -> directory doesn't exist warning -> title")];
+        [alert setInformativeText: [NSString stringWithFormat:
+                NSLocalizedString(@"The directory \"%@\" does not currently exist. "
+                    "Create this directory or choose a different one to create the torrent file.",
+                    "Create torrent -> directory doesn't exist warning -> warning"),
+                    [[fLocation URLByDeletingLastPathComponent] path]]];
+        [alert setAlertStyle: NSWarningAlertStyle];
+        
+        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
+        return;
+    }
+    
     //check if a file with the same name and location already exists
-    if ([[NSFileManager defaultManager] fileExistsAtPath: fLocation])
+    if ([fLocation checkResourceIsReachableAndReturnError: NULL])
     {
         NSArray * pathComponents = [fLocation pathComponents];
         NSInteger count = [pathComponents count];
@@ -413,16 +540,17 @@
     //store values
     [fDefaults setObject: fTrackers forKey: @"CreatorTrackers"];
     [fDefaults setBool: [fPrivateCheck state] == NSOnState forKey: @"CreatorPrivate"];
-    [fDefaults setBool: fOpenTorrent forKey: @"CreatorOpen"];
-    [fDefaults setObject: [fLocation stringByDeletingLastPathComponent] forKey: @"CreatorLocation"];
+    [fDefaults setBool: [fOpenCheck state] == NSOnState forKey: @"CreatorOpen"];
+    fOpenWhenCreated = [fOpenCheck state] == NSOnState; //need this since the check box might not exist, and value in prefs might have changed from another creator window
+    [fDefaults setURL: [fLocation URLByDeletingLastPathComponent] forKey: @"CreatorLocationURL"];
+
+    [[self window] setRestorable: NO];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"BeginCreateTorrentFile" object: fLocation userInfo: nil];
-    tr_makeMetaInfo(fInfo, [fLocation UTF8String], trackerInfo, [fTrackers count], [[fCommentView string] UTF8String],
-                    [fPrivateCheck state] == NSOnState);
+    tr_makeMetaInfo(fInfo, [[fLocation path] UTF8String], trackerInfo, [fTrackers count], [[fCommentView string] UTF8String], [fPrivateCheck state] == NSOnState);
     tr_free(trackerInfo);
     
-    fTimer = [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(checkProgress)
-                userInfo: nil repeats: YES];
+    fTimer = [[NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(checkProgress) userInfo: nil repeats: YES] retain];
 }
 
 - (void) checkProgress
@@ -430,16 +558,17 @@
     if (fInfo->isDone)
     {
         [fTimer invalidate];
+        [fTimer release];
         fTimer = nil;
         
         NSAlert * alert;
         switch (fInfo->result)
         {
             case TR_MAKEMETA_OK:
-                if (fOpenTorrent && [fTrackers count] > 0)
+                if (fOpenWhenCreated)
                 {
-                    NSDictionary * dict = [[NSDictionary alloc] initWithObjectsAndKeys: fLocation, @"File",
-                                            [fPath stringByDeletingLastPathComponent], @"Path", nil];
+                    NSDictionary * dict = [[NSDictionary alloc] initWithObjectsAndKeys: [fLocation path], @"File",
+                                            [[fPath URLByDeletingLastPathComponent] path], @"Path", nil];
                     [[NSNotificationCenter defaultCenter] postNotificationName: @"OpenCreatedTorrentFile" object: self userInfo: dict];
                 }
                 
