@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: DragOverlayWindow.m 8873 2009-08-05 02:13:45Z livings124 $
+ * $Id: DragOverlayWindow.m 14290 2014-06-08 22:16:01Z jordan $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,17 +26,23 @@
 #import "DragOverlayView.h"
 #import "NSStringAdditions.h"
 
+@interface DragOverlayWindow (Private)
+
+- (void) resizeWindow;
+
+@end
+
 @implementation DragOverlayWindow
 
 - (id) initWithLib: (tr_session *) lib forWindow: (NSWindow *) window
 {
-    if ((self = ([super initWithContentRect: NSMakeRect(0, 0, 1.0, 1.0) styleMask: NSBorderlessWindowMask
+    if ((self = ([super initWithContentRect: [window frame] styleMask: NSBorderlessWindowMask
                     backing: NSBackingStoreBuffered defer: NO])))
     {
         fLib = lib;
         
-        [self setBackgroundColor: [NSColor colorWithCalibratedWhite: 0.0f alpha: 0.5f]];
-        [self setAlphaValue: 0.0f];
+        [self setBackgroundColor: [NSColor colorWithCalibratedWhite: 0.0 alpha: 0.5]];
+        [self setAlphaValue: 0.0];
         [self setOpaque: NO];
         [self setHasShadow: NO];
         
@@ -60,12 +66,17 @@
         [fFadeOutAnimation setAnimationBlockingMode: NSAnimationNonblockingThreaded];
         
         [window addChildWindow: self ordered: NSWindowAbove];
+        
+        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(resizeWindow)
+            name: NSWindowDidResizeNotification object: window];
     }
     return self;
 }
 
 - (void) dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
     [fFadeInAnimation release];
     [fFadeOutAnimation release];
     
@@ -99,7 +110,7 @@
                 if (count == 1)
                 {
                     name = [NSString stringWithUTF8String: info.name];
-                    folder = info.isMultifile;
+                    folder = info.isFolder;
                 }
             }
             tr_metainfoFree(&info);
@@ -118,16 +129,18 @@
         if (fileCount == 1)
             fileString = NSLocalizedString(@"1 file", "Drag overlay -> torrents");
         else
-            fileString= [NSString stringWithFormat: NSLocalizedString(@"%d files", "Drag overlay -> torrents"), fileCount];
+            fileString= [NSString stringWithFormat: NSLocalizedString(@"%@ files", "Drag overlay -> torrents"),
+                            [NSString formattedUInteger: fileCount]];
         secondString = [NSString stringWithFormat: @"%@, %@", fileString, secondString];
     }
     
     NSImage * icon;
     if (count == 1)
-        icon = [[NSWorkspace sharedWorkspace] iconForFileType: folder ? NSFileTypeForHFSTypeCode('fldr') : [name pathExtension]];
+        icon = [[NSWorkspace sharedWorkspace] iconForFileType: folder ? NSFileTypeForHFSTypeCode(kGenericFolderIcon) : [name pathExtension]];
     else
     {
-        name = [NSString stringWithFormat: NSLocalizedString(@"%d Torrent Files", "Drag overlay -> torrents"), count];
+        name = [NSString stringWithFormat: NSLocalizedString(@"%@ Torrent Files", "Drag overlay -> torrents"),
+                [NSString formattedUInteger: count]];
         secondString = [secondString stringByAppendingString: @" total"];
         icon = [NSImage imageNamed: @"TransmissionDocument.icns"];
     }
@@ -138,14 +151,14 @@
 
 - (void) setFile: (NSString *) file
 {
-    [[self contentView] setOverlay: [NSImage imageNamed: @"CreateLarge.png"]
+    [[self contentView] setOverlay: [NSImage imageNamed: @"CreateLarge"]
         mainLine: NSLocalizedString(@"Create a Torrent File", "Drag overlay -> file") subLine: file];
     [self fadeIn];
 }
 
 - (void) setURL: (NSString *) url
 {
-    [[self contentView] setOverlay: [NSImage imageNamed: @"Globe.png"]
+    [[self contentView] setOverlay: [NSImage imageNamed: @"Globe"]
         mainLine: NSLocalizedString(@"Web Address", "Drag overlay -> url") subLine: url];
     [self fadeIn];
 }
@@ -158,7 +171,6 @@
         [fFadeOutAnimation stopAnimation];
         [fFadeInAnimation setCurrentProgress: 1.0 - [fFadeOutAnimation currentProgress]];
     }
-    [self setFrame: [[self parentWindow] frame] display: YES];
     [fFadeInAnimation startAnimation];
 }
 
@@ -170,8 +182,17 @@
         [fFadeInAnimation stopAnimation];
         [fFadeOutAnimation setCurrentProgress: 1.0 - [fFadeInAnimation currentProgress]];
     }
-    if ([self alphaValue] > 0.0f)
+    if ([self alphaValue] > 0.0)
         [fFadeOutAnimation startAnimation];
 }
 
 @end
+
+@implementation DragOverlayWindow (Private)
+
+- (void) resizeWindow
+{
+    [self setFrame: [[self parentWindow] frame] display: NO];
+}
+
+@end
