@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: InfoWindowController.h 11876 2011-02-12 03:53:23Z livings124 $
+ * $Id: InfoWindowController.h 14685 2016-02-27 20:29:57Z mikedld $
  *
- * Copyright (c) 2006-2011 Transmission authors and contributors
+ * Copyright (c) 2006-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,7 @@
  *****************************************************************************/
 
 #import <Cocoa/Cocoa.h>
+#import <Quartz/Quartz.h>
 
 @protocol InfoViewController;
 @class InfoGeneralViewController;
@@ -36,6 +37,8 @@
 {
     NSArray * fTorrents;
     
+    CGFloat fMinWindowWidth;
+    
     NSViewController <InfoViewController> * fViewController;
     NSInteger fCurrentTabTag;
     IBOutlet NSMatrix * fTabMatrix;
@@ -62,6 +65,6 @@
 
 - (NSArray *) quickLookURLs;
 - (BOOL) canQuickLook;
-- (NSRect) quickLookSourceFrameForPreviewItem: (id /*<QLPreviewItem>*/) item;
+- (NSRect) quickLookSourceFrameForPreviewItem: (id <QLPreviewItem>) item;
 
 @end
