@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: InfoWindowController.m 12504 2011-06-19 03:52:54Z livings124 $
+ * $Id: InfoWindowController.m 14685 2016-02-27 20:29:57Z mikedld $
  *
- * Copyright (c) 2006-2011 Transmission authors and contributors
+ * Copyright (c) 2006-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -78,7 +78,10 @@ typedef enum
     //window location and size
     NSPanel * window = (NSPanel *)[self window];
     
+    [window setFloatingPanel: NO];
+    
     const CGFloat windowHeight = NSHeight([window frame]);
+    fMinWindowWidth = [window minSize].width;
     
     [window setFrameAutosaveName: @"InspectorWindow"];
     [window setFrameUsingName: @"InspectorWindow"];
@@ -139,12 +142,12 @@ typedef enum
     if ([fViewController respondsToSelector: @selector(saveViewSize)])
         [fViewController saveViewSize];
     
-    [fGeneralViewController dealloc];
-    [fActivityViewController dealloc];
-    [fTrackersViewController dealloc];
-    [fPeersViewController dealloc];
-    [fFileViewController dealloc];
-    [fOptionsViewController dealloc];
+    [fGeneralViewController release];
+    [fActivityViewController release];
+    [fTrackersViewController release];
+    [fPeersViewController release];
+    [fFileViewController release];
+    [fOptionsViewController release];
     
     [fTorrents release];
     
@@ -169,19 +172,10 @@ typedef enum
     return windowRect;
 }
 
-- (NSSize) windowWillResize: (NSWindow *) window toSize: (NSSize) proposedFrameSize
-{
-    //this is an edge-case - just stop the animation
-    [fPeersViewController stopWebSeedAnimation];
-    
-    return proposedFrameSize;
-}
-
 - (void) windowWillClose: (NSNotification *) notification
 {
-    if ([NSApp isOnSnowLeopardOrBetter] && fCurrentTabTag == TAB_FILE_TAG
-        && ([QLPreviewPanelSL sharedPreviewPanelExists] && [[QLPreviewPanelSL sharedPreviewPanel] isVisible]))
-        [[QLPreviewPanelSL sharedPreviewPanel] reloadData];
+    if (fCurrentTabTag == TAB_FILE_TAG && ([QLPreviewPanel sharedPreviewPanelExists] && [[QLPreviewPanel sharedPreviewPanel] isVisible]))
+        [[QLPreviewPanel sharedPreviewPanel] reloadData];
 }
 
 - (void) setTab: (id) sender
@@ -276,7 +270,7 @@ typedef enum
             identifier = TAB_OPTIONS_IDENT;
             break;
         default:
-            NSAssert1(NO, @"Unknown info tab selected: %d", fCurrentTabTag);
+            NSAssert1(NO, @"Unknown info tab selected: %ld", fCurrentTabTag);
             return;
     }
     
@@ -296,18 +290,34 @@ typedef enum
     
     NSRect windowRect = [window frame], viewRect = [view frame];
     
-    CGFloat difference = (NSHeight(viewRect) - oldHeight) * [window userSpaceScaleFactor];
+    const CGFloat difference = NSHeight(viewRect) - oldHeight;
     windowRect.origin.y -= difference;
     windowRect.size.height += difference;
     
+    const CGFloat minWindowWidth = MAX(fMinWindowWidth, [view fittingSize].width);
+    windowRect.size.width = MAX(NSWidth(windowRect), minWindowWidth);
+    
     if ([fViewController respondsToSelector: @selector(saveViewSize)]) //a little bit hacky, but avoids requiring an extra method
     {
-        [window setMinSize: NSMakeSize([window minSize].width, NSHeight(windowRect) - NSHeight(viewRect) + TAB_MIN_HEIGHT)];
+        if ([window screen])
+        {
+            const CGFloat screenHeight = NSHeight([[window screen] visibleFrame]);
+            if (NSHeight(windowRect) > screenHeight)
+            {
+                const CGFloat difference = screenHeight - NSHeight(windowRect);
+                windowRect.origin.y -= difference;
+                windowRect.size.height += difference;
+                
+                viewRect.size.height += difference;
+            }
+        }
+        
+        [window setMinSize: NSMakeSize(minWindowWidth, NSHeight(windowRect) - NSHeight(viewRect) + TAB_MIN_HEIGHT)];
         [window setMaxSize: NSMakeSize(FLT_MAX, FLT_MAX)];
     }
     else
     {
-        [window setMinSize: NSMakeSize([window minSize].width, NSHeight(windowRect))];
+        [window setMinSize: NSMakeSize(minWindowWidth, NSHeight(windowRect))];
         [window setMaxSize: NSMakeSize(FLT_MAX, NSHeight(windowRect))];
     }
     
@@ -317,9 +327,18 @@ typedef enum
     [window setFrame: windowRect display: YES animate: oldTabTag != INVALID];
     [[window contentView] addSubview: view];
     
-    if ([NSApp isOnSnowLeopardOrBetter] && (fCurrentTabTag == TAB_FILE_TAG || oldTabTag == TAB_FILE_TAG)
-        && ([QLPreviewPanelSL sharedPreviewPanelExists] && [[QLPreviewPanelSL sharedPreviewPanel] isVisible]))
-        [[QLPreviewPanelSL sharedPreviewPanel] reloadData];
+    [[window contentView] addConstraints: [NSLayoutConstraint constraintsWithVisualFormat: @"H:|-0-[view]-0-|"
+                                                                                  options: 0
+                                                                                  metrics: nil
+                                                                                    views: @{ @"view": view }]];
+    [[window contentView] addConstraints: [NSLayoutConstraint constraintsWithVisualFormat: @"V:[tabs]-0-[view]-0-|"
+                                                                                  options: 0
+                                                                                  metrics: nil
+                                                                                    views: @{ @"tabs": fTabMatrix, @"view": view }]];
+    
+    if ((fCurrentTabTag == TAB_FILE_TAG || oldTabTag == TAB_FILE_TAG)
+        && ([QLPreviewPanel sharedPreviewPanelExists] && [[QLPreviewPanel sharedPreviewPanel] isVisible]))
+        [[QLPreviewPanel sharedPreviewPanel] reloadData];
 }
 
 - (void) setNextTab
@@ -367,14 +386,13 @@ typedef enum
 
 - (BOOL) canQuickLook
 {
-    if (fCurrentTabTag != TAB_FILE_TAG || ![[self window] isVisible] || ![NSApp isOnSnowLeopardOrBetter])
+    if (fCurrentTabTag != TAB_FILE_TAG || ![[self window] isVisible])
         return NO;
     
     return [fFileViewController canQuickLook];
 }
 
-#warning uncomment (in header too)
-- (NSRect) quickLookSourceFrameForPreviewItem: (id /*<QLPreviewItem>*/) item
+- (NSRect) quickLookSourceFrameForPreviewItem: (id <QLPreviewItem>) item
 {
     return [fFileViewController quickLookSourceFrameForPreviewItem: item];
 }
@@ -436,8 +454,18 @@ typedef enum
                 [fBasicInfoField setStringValue: [NSString stringWithFormat: @"%@, %@", fileString,
                     [NSString stringWithFormat: NSLocalizedString(@"%@ total", "Inspector -> selected torrents"),
                         [NSString stringForFileSize: size]]]];
-                [fBasicInfoField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%@ bytes",
-                                                "Inspector -> selected torrents"), [NSString formattedUInteger: size]]];
+                
+                NSString * byteString;
+                if ([NSApp isOnMountainLionOrBetter])
+                {
+                    NSByteCountFormatter * formatter = [[NSByteCountFormatterMtLion alloc] init];
+                    [formatter setAllowedUnits: NSByteCountFormatterUseBytes];
+                    byteString = [formatter stringFromByteCount: size];
+                    [formatter release];
+                }
+                else
+                    byteString = [NSString stringWithFormat: NSLocalizedString(@"%@ bytes", "Inspector -> selected torrents"), [NSString formattedUInteger: size]];
+                [fBasicInfoField setToolTip: byteString];
             }
             else
             {
@@ -450,8 +478,7 @@ typedef enum
         }
         else
         {
-            #warning change to NSImageNameApplicationIcon
-            [fImageView setImage: [NSImage imageNamed: @"NSApplicationIcon"]];
+            [fImageView setImage: [NSImage imageNamed: NSImageNameApplicationIcon]];
             [fNoneSelectedField setHidden: NO];
             
             [fNameField setHidden: YES];
@@ -464,15 +491,7 @@ typedef enum
     {
         Torrent * torrent = [fTorrents objectAtIndex: 0];
         
-        if ([NSApp isOnSnowLeopardOrBetter])
-            [fImageView setImage: [torrent icon]];
-        else
-        {
-            NSImage * icon = [[torrent icon] copy];
-            [icon setFlipped: NO];
-            [fImageView setImage: icon];
-            [icon release];
-        }
+        [fImageView setImage: [torrent icon]];
         
         NSString * name = [torrent name];
         [fNameField setStringValue: name];
@@ -494,8 +513,18 @@ typedef enum
                 basicString = [NSString stringWithFormat: @"%@, %@", fileString, basicString];
             }
             [fBasicInfoField setStringValue: basicString];
-            [fBasicInfoField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%@ bytes", "Inspector -> selected torrents"),
-                                            [NSString formattedUInteger: [torrent size]]]];
+            
+            NSString * byteString;
+            if ([NSApp isOnMountainLionOrBetter])
+            {
+                NSByteCountFormatter * formatter = [[NSByteCountFormatterMtLion alloc] init];
+                [formatter setAllowedUnits: NSByteCountFormatterUseBytes];
+                byteString = [formatter stringFromByteCount: [torrent size]];
+                [formatter release];
+            }
+            else
+                byteString = [NSString stringWithFormat: NSLocalizedString(@"%@ bytes", "Inspector -> selected torrents"), [NSString formattedUInteger: [torrent size]]];
+            [fBasicInfoField setToolTip: byteString];
         }
         else
         {
@@ -519,7 +548,8 @@ typedef enum
 
 - (void) resetInfoForTorrent: (NSNotification *) notification
 {
-    if (fTorrents && [fTorrents containsObject: [notification object]])
+    Torrent * torrent = [[notification userInfo] objectForKey: @"Torrent"];
+    if (fTorrents && (!torrent || [fTorrents containsObject: torrent]))
         [self resetInfo];
 }
 
