@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: MessageWindowController.m 11617 2011-01-01 20:42:14Z livings124 $
+ * $Id: MessageWindowController.m 14662 2016-01-06 11:05:37Z mikedld $
  *
- * Copyright (c) 2006-2011 Transmission authors and contributors
+ * Copyright (c) 2006-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,10 +23,12 @@
  *****************************************************************************/
 
 #import "MessageWindowController.h"
+#import "Controller.h"
 #import "NSApplicationAdditions.h"
+#import "NSMutableArrayAdditions.h"
 #import "NSStringAdditions.h"
+#import <log.h>
 #import <transmission.h>
-#import <utils.h>
 
 #define LEVEL_ERROR 0
 #define LEVEL_INFO  1
@@ -38,6 +40,7 @@
 
 - (void) resizeColumn;
 - (BOOL) shouldIncludeMessageForFilter: (NSString *) filterString message: (NSDictionary *) message;
+- (void) updateListForFilter;
 - (NSString *) stringForMessage: (NSDictionary *) message;
 
 @end
@@ -49,29 +52,15 @@
     return [super initWithWindowNibName: @"MessageWindow"];
 }
 
-- (void) dealloc
-{
-    [[NSNotificationCenter defaultCenter] removeObserver: self];
-    
-    [fTimer invalidate];
-    [fLock release];
-    
-    [fMessages release];
-    [fDisplayedMessages release];
-    
-    [fAttributes release];
-    
-    [super dealloc];
-}
-
 - (void) awakeFromNib
 {
     NSWindow * window = [self window];
     [window setFrameAutosaveName: @"MessageWindowFrame"];
     [window setFrameUsingName: @"MessageWindowFrame"];
+    [window setRestorationClass: [self class]];
     
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(resizeColumn)
-        name: @"NSTableViewColumnDidResizeNotification" object: fMessageTable];
+        name: NSTableViewColumnDidResizeNotification object: fMessageTable];
     
     [window setContentBorderThickness: NSMinY([[fMessageTable enclosingScrollView] frame]) forEdge: NSMinYEdge];
     
@@ -123,17 +112,17 @@
     //select proper level in popup button
     switch ([[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"])
     {
-        case TR_MSG_ERR:
+        case TR_LOG_ERROR:
             [fLevelButton selectItemAtIndex: LEVEL_ERROR];
             break;
-        case TR_MSG_INF:
+        case TR_LOG_INFO:
             [fLevelButton selectItemAtIndex: LEVEL_INFO];
             break;
-        case TR_MSG_DBG:
+        case TR_LOG_DEBUG:
             [fLevelButton selectItemAtIndex: LEVEL_DEBUG];
             break;
         default: //safety
-            [[NSUserDefaults standardUserDefaults] setInteger: TR_MSG_ERR forKey: @"MessageLevel"];
+            [[NSUserDefaults standardUserDefaults] setInteger: TR_LOG_ERROR forKey: @"MessageLevel"];
             [fLevelButton selectItemAtIndex: LEVEL_ERROR];
     }
     
@@ -143,12 +132,27 @@
     fLock = [[NSLock alloc] init];
 }
 
+- (void) dealloc
+{
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
+    [fTimer invalidate];
+    [fTimer release];
+    [fLock release];
+    
+    [fMessages release];
+    [fDisplayedMessages release];
+    
+    [fAttributes release];
+    
+    [super dealloc];
+}
+
 - (void) windowDidBecomeKey: (NSNotification *) notification
 {
     if (!fTimer)
     {
-        fTimer = [NSTimer scheduledTimerWithTimeInterval: UPDATE_SECONDS target: self
-                    selector: @selector(updateLog:) userInfo: nil repeats: YES];
+        fTimer = [[NSTimer scheduledTimerWithTimeInterval: UPDATE_SECONDS target: self selector: @selector(updateLog:) userInfo: nil repeats: YES] retain];
         [self updateLog: nil];
     }
 }
@@ -156,13 +160,30 @@
 - (void) windowWillClose: (id)sender
 {
     [fTimer invalidate];
+    [fTimer release];
     fTimer = nil;
 }
 
++ (void) restoreWindowWithIdentifier: (NSString *) identifier state: (NSCoder *) state completionHandler: (void (^)(NSWindow *, NSError *)) completionHandler
+{
+    NSAssert1([identifier isEqualToString: @"MessageWindow"], @"Trying to restore unexpected identifier %@", identifier);
+    
+    NSWindow * window = [[(Controller *)[NSApp delegate] messageWindowController] window];
+    completionHandler(window, nil);
+}
+
+- (void) window: (NSWindow *) window didDecodeRestorableState: (NSCoder *) coder
+{
+    [fTimer invalidate];
+    [fTimer release];
+    fTimer = [[NSTimer scheduledTimerWithTimeInterval: UPDATE_SECONDS target: self selector: @selector(updateLog:) userInfo: nil repeats: YES] retain];
+    [self updateLog: nil];
+}
+
 - (void) updateLog: (NSTimer *) timer
 {
-    tr_msg_list * messages;
-    if ((messages = tr_getQueuedMessages()) == NULL)
+    tr_log_message * messages;
+    if ((messages = tr_logGetQueue()) == NULL)
         return;
     
     [fLock lock];
@@ -178,7 +199,7 @@
     
     BOOL changed = NO;
     
-    for (tr_msg_list * currentMessage = messages; currentMessage != NULL; currentMessage = currentMessage->next)
+    for (tr_log_message * currentMessage = messages; currentMessage != NULL; currentMessage = currentMessage->next)
     {
         NSString * name = currentMessage->name != NULL ? [NSString stringWithUTF8String: currentMessage->name]
                             : [[NSProcessInfo processInfo] processName];
@@ -203,11 +224,11 @@
         }
     }
     
-    if ([fMessages count] > TR_MAX_MSG_LOG)
+    if ([fMessages count] > TR_LOG_MAX_QUEUE_LENGTH)
     {
         const NSUInteger oldCount = [fDisplayedMessages count];
         
-        NSIndexSet * removeIndexes = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fMessages count]-TR_MAX_MSG_LOG)];
+        NSIndexSet * removeIndexes = [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fMessages count]-TR_LOG_MAX_QUEUE_LENGTH)];
         NSArray * itemsToRemove = [fMessages objectsAtIndexes: removeIndexes];
         
         [fMessages removeObjectsAtIndexes: removeIndexes];
@@ -227,7 +248,7 @@
     
     [fLock unlock];
     
-    tr_freeMessageList(messages);
+    tr_logFreeQueue (messages);
 }
 
 - (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView
@@ -247,14 +268,14 @@
         const NSInteger level = [[message objectForKey: @"Level"] integerValue];
         switch (level)
         {
-            case TR_MSG_ERR:
-                return [NSImage imageNamed: @"RedDot.png"];
-            case TR_MSG_INF:
-                return [NSImage imageNamed: @"YellowDot.png"];
-            case TR_MSG_DBG:
-                return [NSImage imageNamed: @"PurpleDot.png"];
+            case TR_LOG_ERROR:
+                return [NSImage imageNamed: @"RedDot"];
+            case TR_LOG_INFO:
+                return [NSImage imageNamed: @"YellowDot"];
+            case TR_LOG_DEBUG:
+                return [NSImage imageNamed: @"PurpleDot"];
             default:
-                NSAssert1(NO, @"Unknown message log level: %d", level);
+                NSAssert1(NO, @"Unknown message log level: %ld", level);
                 return nil;
         }
     }
@@ -271,6 +292,7 @@
     
     NSTableColumn * column = [tableView tableColumnWithIdentifier: @"Message"];
     const CGFloat count = floorf([message sizeWithAttributes: fAttributes].width / [column width]);
+    
     return [tableView rowHeight] * (count + 1.0);
 }
 
@@ -298,16 +320,8 @@
     NSString * messageString = [messageStrings componentsJoinedByString: @"\n"];
     
     NSPasteboard * pb = [NSPasteboard generalPasteboard];
-    if ([NSApp isOnSnowLeopardOrBetter])
-    {
-        [pb clearContents];
-        [pb writeObjects: [NSArray arrayWithObject: messageString]];
-    }
-    else
-    {
-        [pb declareTypes: [NSArray arrayWithObject: NSStringPboardType] owner: nil];
-        [pb setString: messageString forType: NSStringPboardType];
-    }
+    [pb clearContents];
+    [pb writeObjects: [NSArray arrayWithObject: messageString]];
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem *) menuItem
@@ -326,42 +340,26 @@
     switch ([fLevelButton indexOfSelectedItem])
     {
         case LEVEL_ERROR:
-            level = TR_MSG_ERR;
+            level = TR_LOG_ERROR;
             break;
         case LEVEL_INFO:
-            level = TR_MSG_INF;
+            level = TR_LOG_INFO;
             break;
         case LEVEL_DEBUG:
-            level = TR_MSG_DBG;
+            level = TR_LOG_DEBUG;
             break;
         default:
-            NSAssert1(NO, @"Unknown message log level: %d", [fLevelButton indexOfSelectedItem]);
+            NSAssert1(NO, @"Unknown message log level: %ld", [fLevelButton indexOfSelectedItem]);
     }
     
     if ([[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"] == level)
         return;
     
-    [fLock lock];
-    
     [[NSUserDefaults standardUserDefaults] setInteger: level forKey: @"MessageLevel"];
     
-    NSString * filterString = [fFilterField stringValue];
+    [fLock lock];
     
-    [fDisplayedMessages removeAllObjects];
-    for (NSDictionary * message in fMessages)
-        if ([[message objectForKey: @"Level"] integerValue] <= level
-            && [self shouldIncludeMessageForFilter: filterString message: message])
-            [fDisplayedMessages addObject: message];
-    
-    [fDisplayedMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
-    
-    [fMessageTable reloadData];
-    
-    if ([fDisplayedMessages count] > 0)
-    {
-        [fMessageTable deselectAll: self];
-        [fMessageTable scrollRowToVisible: [fMessageTable numberOfRows]-1];
-    }
+    [self updateListForFilter];
     
     [fLock unlock];
 }
@@ -370,24 +368,7 @@
 {
     [fLock lock];
     
-    const NSInteger level = [[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"];
-    NSString * filterString = [fFilterField stringValue];
-    
-    [fDisplayedMessages removeAllObjects];
-    for (NSDictionary * message in fMessages)
-        if ([[message objectForKey: @"Level"] integerValue] <= level
-            && [self shouldIncludeMessageForFilter: filterString message: message])
-            [fDisplayedMessages addObject: message];
-    
-    [fDisplayedMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
-    
-    [fMessageTable reloadData];
-    
-    if ([fDisplayedMessages count] > 0)
-    {
-        [fMessageTable deselectAll: self];
-        [fMessageTable scrollRowToVisible: [fMessageTable numberOfRows]-1];
-    }
+    [self updateListForFilter];
     
     [fLock unlock];
 }
@@ -397,56 +378,56 @@
     [fLock lock];
     
     [fMessages removeAllObjects];
+
+    [fMessageTable beginUpdates];
+    [fMessageTable removeRowsAtIndexes: [NSIndexSet indexSetWithIndexesInRange: NSMakeRange(0, [fDisplayedMessages count])] withAnimation: NSTableViewAnimationSlideLeft];
+
     [fDisplayedMessages removeAllObjects];
-    [fMessageTable reloadData];
+
+    [fMessageTable endUpdates];
     
     [fLock unlock];
 }
 
 - (void) writeToFile: (id) sender
 {
-    //make the array sorted by date
-    NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Index" ascending: YES] autorelease];
-    NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
-    NSArray * sortedMessages = [[fDisplayedMessages sortedArrayUsingDescriptors: descriptors] retain];
-    [descriptors release];
-    
     NSSavePanel * panel = [NSSavePanel savePanel];
-    [panel setRequiredFileType: @"txt"];
+    [panel setAllowedFileTypes: [NSArray arrayWithObject: @"txt"]];
     [panel setCanSelectHiddenExtension: YES];
     
-    [panel beginSheetForDirectory: nil file: NSLocalizedString(@"untitled", "Save log panel -> default file name")
-            modalForWindow: [self window] modalDelegate: self
-            didEndSelector: @selector(writeToFileSheetClosed:returnCode:contextInfo:) contextInfo: sortedMessages];
-}
-
-- (void) writeToFileSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (NSArray *) messages
-{
-    if (code == NSOKButton)
-    {
-        //create the text to output
-        NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [messages count]];
-        for (NSDictionary * message in messages)
-            [messageStrings addObject: [self stringForMessage: message]];
+    [panel setNameFieldStringValue: NSLocalizedString(@"untitled", "Save log panel -> default file name")];
     
-        NSString * fileString = [messageStrings componentsJoinedByString: @"\n"];
-        
-        if (![fileString writeToFile: [panel filename] atomically: YES encoding: NSUTF8StringEncoding error: nil])
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        if (result == NSFileHandlingPanelOKButton)
         {
-            NSAlert * alert = [[NSAlert alloc] init];
-            [alert addButtonWithTitle: NSLocalizedString(@"OK", "Save log alert panel -> button")];
-            [alert setMessageText: NSLocalizedString(@"Log Could Not Be Saved", "Save log alert panel -> title")];
-            [alert setInformativeText: [NSString stringWithFormat:
-                    NSLocalizedString(@"There was a problem creating the file \"%@\".",
-                    "Save log alert panel -> message"), [[panel filename] lastPathComponent]]];
-            [alert setAlertStyle: NSWarningAlertStyle];
+            //make the array sorted by date
+            NSSortDescriptor * descriptor = [NSSortDescriptor sortDescriptorWithKey: @"Index" ascending: YES];
+            NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
+            NSArray * sortedMessages = [fDisplayedMessages sortedArrayUsingDescriptors: descriptors];
+            [descriptors release];
             
-            [alert runModal];
-            [alert release];
+            //create the text to output
+            NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [sortedMessages count]];
+            for (NSDictionary * message in sortedMessages)
+                [messageStrings addObject: [self stringForMessage: message]];
+            
+            NSString * fileString = [messageStrings componentsJoinedByString: @"\n"];
+            
+            if (![fileString writeToFile: [[panel URL] path] atomically: YES encoding: NSUTF8StringEncoding error: nil])
+            {
+                NSAlert * alert = [[NSAlert alloc] init];
+                [alert addButtonWithTitle: NSLocalizedString(@"OK", "Save log alert panel -> button")];
+                [alert setMessageText: NSLocalizedString(@"Log Could Not Be Saved", "Save log alert panel -> title")];
+                [alert setInformativeText: [NSString stringWithFormat:
+                                            NSLocalizedString(@"There was a problem creating the file \"%@\".",
+                                                              "Save log alert panel -> message"), [[[panel URL] path] lastPathComponent]]];
+                [alert setAlertStyle: NSWarningAlertStyle];
+                
+                [alert runModal];
+                [alert release];
+            }
         }
-    }
-    
-    [messages release];
+    }];
 }
 
 @end
@@ -469,23 +450,79 @@
             || [[message objectForKey: @"Message"] rangeOfString: filterString options: searchOptions].location != NSNotFound;
 }
 
+- (void) updateListForFilter
+{
+    const NSInteger level = [[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"];
+    NSString * filterString = [fFilterField stringValue];
+    
+    NSIndexSet * indexes = [fMessages indexesOfObjectsWithOptions: NSEnumerationConcurrent passingTest: ^BOOL(id message, NSUInteger idx, BOOL * stop) {
+        return [[(NSDictionary *)message objectForKey: @"Level"] integerValue] <= level && [self shouldIncludeMessageForFilter: filterString message: message];
+    }];
+    
+    NSArray * tempMessages = [[fMessages objectsAtIndexes: indexes] sortedArrayUsingDescriptors: [fMessageTable sortDescriptors]];
+
+    [fMessageTable beginUpdates];
+    
+    //figure out which rows were added/moved
+    NSUInteger currentIndex = 0, totalCount = 0;
+    NSMutableArray * itemsToAdd = [NSMutableArray array];
+    NSMutableIndexSet * itemsToAddIndexes = [NSMutableIndexSet indexSet];
+    
+    for (NSDictionary * message in tempMessages)
+    {
+        const NSUInteger previousIndex = [fDisplayedMessages indexOfObject: message inRange: NSMakeRange(currentIndex, [fDisplayedMessages count]-currentIndex)];
+        if (previousIndex == NSNotFound)
+        {
+            [itemsToAdd addObject: message];
+            [itemsToAddIndexes addIndex: totalCount];
+        }
+        else
+        {
+            if (previousIndex != currentIndex)
+            {
+                [fDisplayedMessages moveObjectAtIndex: previousIndex toIndex: currentIndex];
+                [fMessageTable moveRowAtIndex: previousIndex toIndex: currentIndex];
+            }
+            ++currentIndex;
+        }
+        
+        ++totalCount;
+    }
+    
+    //remove trailing items - those are the unused
+    if (currentIndex < [fDisplayedMessages count])
+    {
+        const NSRange removeRange = NSMakeRange(currentIndex, [fDisplayedMessages count]-currentIndex);
+        [fDisplayedMessages removeObjectsInRange: removeRange];
+        [fMessageTable removeRowsAtIndexes: [NSIndexSet indexSetWithIndexesInRange: removeRange] withAnimation: NSTableViewAnimationSlideDown];
+    }
+    
+    //add new items
+    [fDisplayedMessages insertObjects: itemsToAdd atIndexes: itemsToAddIndexes];
+    [fMessageTable insertRowsAtIndexes: itemsToAddIndexes withAnimation: NSTableViewAnimationSlideUp];
+
+    [fMessageTable endUpdates];
+    
+    NSAssert2([fDisplayedMessages isEqualToArray: tempMessages], @"Inconsistency between message arrays! %@ %@", fDisplayedMessages, tempMessages);
+}
+
 - (NSString *) stringForMessage: (NSDictionary *) message
 {
     NSString * levelString;
     const NSInteger level = [[message objectForKey: @"Level"] integerValue];
     switch (level)
     {
-        case TR_MSG_ERR:
+        case TR_LOG_ERROR:
             levelString = NSLocalizedString(@"Error", "Message window -> level");
             break;
-        case TR_MSG_INF:
+        case TR_LOG_INFO:
             levelString = NSLocalizedString(@"Info", "Message window -> level");
             break;
-        case TR_MSG_DBG:
+        case TR_LOG_DEBUG:
             levelString = NSLocalizedString(@"Debug", "Message window -> level");
             break;
         default:
-            NSAssert1(NO, @"Unknown message log level: %d", level);
+            NSAssert1(NO, @"Unknown message log level: %ld", level);
     }
     
     return [NSString stringWithFormat: @"%@ %@ [%@] %@: %@", [message objectForKey: @"Date"],
