@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: NSApplicationAdditions.m 11617 2011-01-01 20:42:14Z livings124 $
+ * $Id: NSApplicationAdditions.m 14662 2016-01-06 11:05:37Z mikedld $
  *
- * Copyright (c) 2009-2011 Transmission authors and contributors
+ * Copyright (c) 2009-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,9 +26,14 @@
 
 @implementation NSApplication (NSApplicationAdditions)
 
-- (BOOL) isOnSnowLeopardOrBetter
+- (BOOL) isOnMountainLionOrBetter
 {
-	return floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5;
+    return floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_7;
+}
+
+- (BOOL) isOnYosemiteOrBetter
+{
+    return floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_9;
 }
 
 @end
