@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Torrent.h 12325 2011-04-05 23:03:33Z livings124 $
+ * $Id: Torrent.h 14587 2015-10-23 04:09:40Z mikedld $
  *
- * Copyright (c) 2006-2011 Transmission authors and contributors
+ * Copyright (c) 2006-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,8 +28,14 @@
 
 @class FileListNode;
 
-#warning uncomment
-@interface Torrent : NSObject <NSCopying>//, QLPreviewItem>
+typedef enum {
+	TorrentDeterminationAutomatic = 0,
+	TorrentDeterminationUserSpecified
+} TorrentDeterminationType;
+
+#define kTorrentDidChangeGroupNotification @"TorrentDidChangeGroup"
+
+@interface Torrent : NSObject <NSCopying, QLPreviewItem>
 {
     tr_torrent * fHandle;
     const tr_info * fInfo;
@@ -47,13 +53,16 @@
     NSIndexSet * fPreviousFinishedIndexes;
     NSDate * fPreviousFinishedIndexesDate;
     
-    BOOL fWaitToStart, fStalled;
+    BOOL fRemoveWhenFinishSeeding;
     
     NSInteger fGroupValue;
+	TorrentDeterminationType fGroupValueDetermination;
+	
+	TorrentDeterminationType fDownloadFolderDetermination;
     
     BOOL fResumeOnWake;
     
-    NSString * fTimeMachineExclude;
+    BOOL fTimeMachineExcludeInitialized;
 }
 
 - (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (BOOL) torrentDelete
@@ -66,22 +75,27 @@
 
 - (void) closeRemoveTorrent: (BOOL) trashFiles;
 
-- (void) changeDownloadFolderBeforeUsing: (NSString *) folder;
+- (void) changeDownloadFolderBeforeUsing: (NSString *) folder determinationType: (TorrentDeterminationType) determinationType;
 
 - (NSString *) currentDirectory;
 
 - (void) getAvailability: (int8_t *) tab size: (NSInteger) size;
 - (void) getAmountFinished: (float *) tab size: (NSInteger) size;
 - (NSIndexSet *) previousFinishedPieces;
--(void) setPreviousFinishedPieces: (NSIndexSet *) indexes;
+- (void) setPreviousFinishedPieces: (NSIndexSet *) indexes;
 
 - (void) update;
 
+- (void) startTransferIgnoringQueue: (BOOL) ignoreQueue;
+- (void) startTransferNoQueue;
 - (void) startTransfer;
 - (void) stopTransfer;
 - (void) sleep;
 - (void) wakeUp;
 
+- (NSInteger) queuePosition;
+- (void) setQueuePosition: (NSUInteger) index;
+
 - (void) manualAnnounce;
 - (BOOL) canManualAnnounce;
 
@@ -112,13 +126,14 @@
 - (void) setMaxPeerConnect: (uint16_t) count;
 - (uint16_t) maxPeerConnect;
 
-- (void) setWaitToStart: (BOOL) wait;
+@property (nonatomic) BOOL removeWhenFinishSeeding;
+
 - (BOOL) waitingToStart;
 
 - (tr_priority_t) priority;
 - (void) setPriority: (tr_priority_t) priority;
 
-+ (void) trashFile: (NSString *) path;
++ (BOOL) trashFile: (NSString *) path error: (NSError **) error;
 - (void) moveTorrentDataFileTo: (NSString *) folder;
 - (void) copyTorrentFileTo: (NSString *) path;
 
@@ -149,6 +164,9 @@
 - (NSString *) dataLocation;
 - (NSString *) fileLocation: (FileListNode *) node;
 
+- (void) renameTorrent: (NSString *) newName completionHandler: (void (^)(BOOL didRename)) completionHandler;
+- (void) renameFileNode: (FileListNode *) node withName: (NSString *) newName completionHandler: (void (^)(BOOL didRename)) completionHandler;
+
 - (CGFloat) progress;
 - (CGFloat) progressDone;
 - (CGFloat) progressLeft;
@@ -201,7 +219,7 @@
 - (uint64_t) failedHash;
 
 - (NSInteger) groupValue;
-- (void) setGroupValue: (NSInteger) groupValue;
+- (void) setGroupValue: (NSInteger) groupValue determinationType: (TorrentDeterminationType) determinationType;;
 - (NSInteger) groupOrderValue;
 - (void) checkGroupValueForRemoval: (NSNotification *) notification;
 
