@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentTableView.m 12496 2011-06-13 22:26:43Z livings124 $
+ * $Id: TorrentTableView.m 14705 2016-03-03 17:57:08Z mikedld $
  *
- * Copyright (c) 2005-2011 Transmission authors and contributors
+ * Copyright (c) 2005-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,7 @@
 #import "TorrentTableView.h"
 #import "Controller.h"
 #import "FileListNode.h"
+#import "InfoOptionsViewController.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "Torrent.h"
@@ -33,6 +34,7 @@
 
 #define MAX_GROUP 999999
 
+//eliminate when Lion-only
 #define ACTION_MENU_GLOBAL_TAG 101
 #define ACTION_MENU_UNLIMITED_TAG 102
 #define ACTION_MENU_LIMIT_TAG 103
@@ -71,7 +73,8 @@
         fMouseControlRow = -1;
         fMouseRevealRow = -1;
         fMouseActionRow = -1;
-        fActionPushedRow = -1;
+        
+        fActionPopoverShown = NO;
         
         [self setDelegate: self];
         
@@ -83,6 +86,8 @@
 
 - (void) dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
     [fCollapsedGroups release];
     
     [fPiecesBarAnimation release];
@@ -99,6 +104,8 @@
 {
     //set group columns to show ratio, needs to be in awakeFromNib to size columns correctly
     [self setGroupStatusColumns];
+    
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(setNeedsDisplay) name: @"RefreshTorrentTable" object: nil];
 }
 
 - (BOOL) isGroupCollapsed: (NSInteger) value
@@ -151,14 +158,16 @@
 {
     if ([item isKindOfClass: [Torrent class]])
     {
-        [cell setRepresentedObject: item];
-        
-        const NSInteger row = [self rowForItem: item];
-        [cell setHover: row == fMouseRow];
-        [cell setControlHover: row == fMouseControlRow];
-        [cell setRevealHover: row == fMouseRevealRow];
-        [cell setActionHover: row == fMouseActionRow];
-        [cell setActionPushed: row == fActionPushedRow];
+        if (!tableColumn)
+        {
+            [cell setRepresentedObject: item];
+            
+            const NSInteger row = [self rowForItem: item];
+            [cell setHover: row == fMouseRow];
+            [cell setControlHover: row == fMouseControlRow];
+            [cell setRevealHover: row == fMouseRevealRow];
+            [cell setActionHover: row == fMouseActionRow];
+        }
     }
     else
     {
@@ -166,19 +175,7 @@
         if ([ident isEqualToString: @"UL Image"] || [ident isEqualToString: @"DL Image"])
         {
             //ensure arrows are white only when selected
-            if ([NSApp isOnSnowLeopardOrBetter])
-                [[cell image] setTemplate: [cell backgroundStyle] == NSBackgroundStyleLowered];
-            else
-            {
-                NSImage * image = [cell image];
-                const BOOL template = [cell backgroundStyle] == NSBackgroundStyleLowered;
-                if ([image isTemplate] != template)
-                {
-                    [image setTemplate: template];
-                    [cell setImage: nil];
-                    [cell setImage: image];
-                }
-            }
+            [[cell image] setTemplate: [cell backgroundStyle] == NSBackgroundStyleLowered];
         }
     }
 }
@@ -201,12 +198,13 @@
 
 - (NSString *) outlineView: (NSOutlineView *) outlineView typeSelectStringForTableColumn: (NSTableColumn *) tableColumn item: (id) item
 {
-    return [item isKindOfClass: [Torrent class]] ? [item name]
-            : [[self preparedCellAtColumn: [self columnWithIdentifier: @"Group"] row: [self rowForItem: item]] stringValue];
+    if ([item isKindOfClass: [Torrent class]])
+        return [(Torrent *)item name];
+    else
+        return [[self dataSource] outlineView:outlineView objectValueForTableColumn:[self tableColumnWithIdentifier:@"Group"] byItem:item];
 }
 
-- (NSString *) outlineView: (NSOutlineView *) outlineView toolTipForCell: (NSCell *) cell rect: (NSRectPointer) rect
-                tableColumn: (NSTableColumn *) column item: (id) item mouseLocation: (NSPoint) mouseLocation
+- (NSString *) outlineView: (NSOutlineView *) outlineView toolTipForCell: (NSCell *) cell rect: (NSRectPointer) rect tableColumn: (NSTableColumn *) column item: (id) item mouseLocation: (NSPoint) mouseLocation
 {
     NSString * ident = [column identifier];
     if ([ident isEqualToString: @"DL"] || [ident isEqualToString: @"DL Image"])
@@ -236,7 +234,7 @@
     if (rows.length == 0)
         return;
     
-    NSPoint mouseLocation = [self convertPoint: [[self window] convertScreenToBase: [NSEvent mouseLocation]] fromView: nil];
+    NSPoint mouseLocation = [self convertPoint: [[self window] mouseLocationOutsideOfEventStream] fromView: nil];
     for (NSUInteger row = rows.location; row < NSMaxRange(rows); row++)
     {
         if (![[self itemAtRow: row] isKindOfClass: [Torrent class]])
@@ -345,6 +343,8 @@
 
 - (void) outlineViewSelectionIsChanging: (NSNotification *) notification
 {
+    #warning elliminate when view-based?
+    //if pushing a button, don't change the selected rows
     if (fSelectedValues)
         [self selectValues: fSelectedValues];
 }
@@ -400,13 +400,8 @@
     //avoid weird behavior when showing menu by doing this after mouse down
     if (row != -1 && fMouseActionRow == row)
     {
-        fActionPushedRow = row;
-        [self setNeedsDisplayInRect: [self rectOfRow: row]]; //ensure button is pushed down
-        
-        [self displayTorrentMenuForEvent: event];
-        
-        fActionPushedRow = -1;
-        [self setNeedsDisplayInRect: [self rectOfRow: row]];
+        #warning maybe make appear on mouse down
+        [self displayTorrentActionPopoverForEvent: event];
     }
     else if (!pushed && [event clickCount] == 2) //double click
     {
@@ -522,6 +517,8 @@
         [fController focusFilterField];
     else if (firstChar == ' ')
         [fController toggleQuickLook: nil];
+    else if ([event keyCode] == 53) //esc key
+        [self deselectAll: nil];
     else
         [super keyDown: event];
 }
@@ -531,12 +528,31 @@
     return [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
 }
 
-#warning catch string urls?
 - (void) paste: (id) sender
 {
     NSURL * url;
     if ((url = [NSURL URLFromPasteboard: [NSPasteboard generalPasteboard]]))
         [fController openURL: [url absoluteString]];
+    else
+    {
+        NSArray * items = [[NSPasteboard generalPasteboard] readObjectsForClasses: [NSArray arrayWithObject: [NSString class]] options: nil];
+        if (items)
+        {
+            NSDataDetector * detector = [NSDataDetector dataDetectorWithTypes: NSTextCheckingTypeLink error: nil];
+            for (NSString * pbItem in items)
+            {
+                pbItem = [pbItem stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+                if ([pbItem rangeOfString: @"magnet:" options: (NSAnchoredSearch | NSCaseInsensitiveSearch)].location != NSNotFound)
+                    [fController openURL: pbItem];
+                else
+                {
+                    #warning only accept full text?
+                    for (NSTextCheckingResult * result in [detector matchesInString: pbItem options: 0 range: NSMakeRange(0, [pbItem length])])
+                        [fController openURL: [[result URL] absoluteString]];
+                }
+            }
+        }
+    }
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem *) menuItem
@@ -544,7 +560,25 @@
     SEL action = [menuItem action];
     
     if (action == @selector(paste:))
-        return [[[NSPasteboard generalPasteboard] types] containsObject: NSURLPboardType];
+    {
+        if ([[[NSPasteboard generalPasteboard] types] containsObject: NSURLPboardType])
+            return YES;
+
+        NSArray * items = [[NSPasteboard generalPasteboard] readObjectsForClasses: [NSArray arrayWithObject: [NSString class]] options: nil];
+        if (items)
+        {
+            NSDataDetector * detector = [NSDataDetector dataDetectorWithTypes: NSTextCheckingTypeLink error: nil];
+            for (NSString * pbItem in items)
+            {
+                pbItem = [pbItem stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+                if (([pbItem rangeOfString: @"magnet:" options: (NSAnchoredSearch | NSCaseInsensitiveSearch)].location != NSNotFound)
+                    || [detector firstMatchInString: pbItem options: 0 range: NSMakeRange(0, [pbItem length])])
+                    return YES;
+            }
+        }
+    
+    return NO;
+    }
     
     return YES;
 }
@@ -555,7 +589,7 @@
         [fController stopTorrents: [NSArray arrayWithObject: torrent]];
     else
     {
-        if (([NSApp isOnSnowLeopardOrBetter] ? [NSEvent modifierFlags] : [[NSApp currentEvent] modifierFlags]) & NSAlternateKeyMask)
+        if ([NSEvent modifierFlags] & NSAlternateKeyMask)
             [fController resumeTorrentsNoWait: [NSArray arrayWithObject: torrent]];
         else if ([torrent waitingToStart])
             [fController stopTorrents: [NSArray arrayWithObject: torrent]];
@@ -564,42 +598,45 @@
     }
 }
 
-- (void) displayTorrentMenuForEvent: (NSEvent *) event
+- (void) displayTorrentActionPopoverForEvent: (NSEvent *) event
 {
     const NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
     if (row < 0)
         return;
     
-    //update file action menu
-    fMenuTorrent = [[self itemAtRow: row] retain];
+    const NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
+
+    if (fActionPopoverShown)
+        return;
     
-    //update global limit check
-    [fGlobalLimitItem setState: [fMenuTorrent usesGlobalSpeedLimit] ? NSOnState : NSOffState];
+    Torrent * torrent = [self itemAtRow: row];
     
-    //place menu below button
-    NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
-    NSPoint location = rect.origin;
-    location.y += rect.size.height + 5.0;
+    NSPopover * popover = [[NSPopover alloc] init];
+    [popover setBehavior: NSPopoverBehaviorTransient];
+    InfoOptionsViewController * infoViewController = [[InfoOptionsViewController alloc] init];
+    [popover setContentViewController: infoViewController];
+    [popover setDelegate: self];
     
-    if ([NSApp isOnSnowLeopardOrBetter])
-    {
-        location = [self convertPoint: location toView: self];
-        [fActionMenu popUpMenuPositioningItem: nil atLocation: location inView: self];
-    }
-    else
-    {
-        location = [self convertPoint: location toView: nil];
-        NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
-            modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
-            context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
-        
-        [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
-    }
+    [popover showRelativeToRect: rect ofView: self preferredEdge: NSMaxYEdge];
+    [infoViewController setInfoForTorrents: [NSArray arrayWithObject: torrent]];
+    [infoViewController updateInfo];
     
-    [fMenuTorrent release];
-    fMenuTorrent = nil;
+    [infoViewController release];
+    [popover release];
 }
 
+//don't show multiple popovers when clicking the gear button repeatedly
+- (void) popoverWillShow: (NSNotification *) notification
+{
+    fActionPopoverShown = YES;
+}
+
+- (void) popoverWillClose: (NSNotification *) notification
+{
+    fActionPopoverShown = NO;
+}
+
+//eliminate when Lion-only, along with all the menu item instance variables
 - (void) menuNeedsUpdate: (NSMenu *) menu
 {
     //this method seems to be called when it shouldn't be
@@ -683,6 +720,7 @@
     }
 }
 
+//the following methods might not be needed when Lion-only
 - (void) setQuickLimitMode: (id) sender
 {
     const BOOL limit = [sender tag] == ACTION_MENU_LIMIT_TAG;
@@ -702,7 +740,7 @@
 
 - (void) setGlobalLimit: (id) sender
 {
-    [fMenuTorrent setUseGlobalSpeedLimit: [sender state] != NSOnState];
+    [fMenuTorrent setUseGlobalSpeedLimit: [(NSButton *)sender state] != NSOnState];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
@@ -753,7 +791,7 @@
             priority = TR_PRI_LOW;
             break;
         default:
-            NSAssert1(NO, @"Unknown priority: %d", [sender tag]);
+            NSAssert1(NO, @"Unknown priority: %ld", [sender tag]);
     }
     
     [fMenuTorrent setPriority: priority];
@@ -763,14 +801,12 @@
 
 - (void) togglePiecesBar
 {
-    //stop previous animation
-    if (fPiecesBarAnimation)
-        [fPiecesBarAnimation release];
-    
     NSMutableArray * progressMarks = [NSMutableArray arrayWithCapacity: 16];
     for (NSAnimationProgress i = 0.0625; i <= 1.0; i += 0.0625)
-        [progressMarks addObject: [NSNumber numberWithDouble: i]];
+        [progressMarks addObject: [NSNumber numberWithFloat: i]];
     
+    //this stops a previous animation
+    [fPiecesBarAnimation release];
     fPiecesBarAnimation = [[NSAnimation alloc] initWithDuration: TOGGLE_PROGRESS_SECONDS animationCurve: NSAnimationEaseIn];
     [fPiecesBarAnimation setAnimationBlockingMode: NSAnimationNonblocking];
     [fPiecesBarAnimation setProgressMarks: progressMarks];
@@ -797,7 +833,7 @@
         else
             fPiecesBarPercent = 1.0 - progress;
         
-        [self reloadData];
+        [self setNeedsDisplay: YES];
     }
 }
 
@@ -806,6 +842,24 @@
     return fPiecesBarPercent;
 }
 
+- (void) selectAndScrollToRow: (NSInteger) row
+{
+    NSParameterAssert(row >= 0);
+    NSParameterAssert(row < [self numberOfRows]);
+    
+    [self selectRowIndexes: [NSIndexSet indexSetWithIndex: row] byExtendingSelection: NO];
+    
+    const NSRect rowRect = [self rectOfRow: row];
+    const NSRect viewRect = [[self superview] frame];
+
+    NSPoint scrollOrigin = rowRect.origin;
+    scrollOrigin.y += (rowRect.size.height - viewRect.size.height) / 2;
+    if (scrollOrigin.y < 0)
+        scrollOrigin.y = 0;
+
+    [[[self superview] animator] setBoundsOrigin: scrollOrigin];
+}
+
 @end
 
 @implementation TorrentTableView (Private)
